# -*- coding: utf-8 -*-

import logging
from random import shuffle

from claw.constants import RE_DELIMITER


log = logging.getLogger(__name__)


def safe_format(format_string, *args, **kwargs):
    """
    Helper: formats string with any combination of bytestrings/unicode
    strings without raising exceptions
    """
    try:
        if not args and not kwargs:
            return format_string
        else:
            return format_string.format(*args, **kwargs)

    # catch encoding errors and transform everything into utf-8 string
    # before logging:
    except (UnicodeEncodeError, UnicodeDecodeError):
        format_string = to_utf8(format_string)
        args = [to_utf8(p) for p in args]
        kwargs = {k: to_utf8(v) for k, v in kwargs.iteritems()}
        return format_string.format(*args, **kwargs)

    # ignore other errors
    except:
        return u''


def to_unicode(str_or_unicode, precise=False):
    """
    Safely returns a unicode version of a given string
    >>> utils.to_unicode('привет')
        u'привет'
    >>> utils.to_unicode(u'привет')
        u'привет'
    If `precise` flag is True, tries to guess the correct encoding first.
    """
    encoding = detect_encoding(str_or_unicode) if precise else 'utf-8'
    if isinstance(str_or_unicode, str):
        return unicode(str_or_unicode, encoding, 'replace')
    return str_or_unicode


def to_utf8(str_or_unicode):
    """
    Safely returns a UTF-8 version of a given string
    >>> utils.to_utf8(u'hi')
        'hi'
    """
    if isinstance(str_or_unicode, unicode):
        return str_or_unicode.encode("utf-8", "ignore")
    return str(str_or_unicode)


def random_token(length=7):
    vals = ("a b c d e f g h i j k l m n o p q r s t u v w x y z "
            "0 1 2 3 4 5 6 7 8 9").split(' ')
    shuffle(vals)
    return ''.join(vals[:length])


def get_delimiter(msg_body):
    delimiter = RE_DELIMITER.search(msg_body)
    if delimiter:
        delimiter = delimiter.group()
    else:
        delimiter = '\n'

    return delimiter



# -*- coding: utf-8 -*-

from mock import Mock, patch
from nose.tools import *


from claw.signature import bruteforce


def test_empty_body():
    eq_(('', None), bruteforce.extract_signature(''))


def test_no_signature():
    msg_body = 'Hey man!'
    eq_((msg_body, None), bruteforce.extract_signature(msg_body))


def test_signature_only():
    msg_body = '--\nRoman'
    eq_((msg_body, None), bruteforce.extract_signature(msg_body))


def test_signature_separated_by_dashes():
    msg_body = '''Hey man! How r u?
---
Roman'''
    eq_(('Hey man! How r u?', '---\nRoman'),
        bruteforce.extract_signature(msg_body))

    msg_body = '''Hey!
-roman'''
    eq_(('Hey!', '-roman'), bruteforce.extract_signature(msg_body))

    msg_body = '''Hey!

- roman'''
    eq_(('Hey!', '- roman'), bruteforce.extract_signature(msg_body))

    msg_body = '''Wow. Awesome!
--
Bob Smith'''
    eq_(('Wow. Awesome!', '--\nBob Smith'),
        bruteforce.extract_signature(msg_body))


def test_signature_words():
    msg_body = '''Hey!

Thanks!
Roman'''
    eq_(('Hey!', 'Thanks!\nRoman'),
        bruteforce.extract_signature(msg_body))

    msg_body = '''Hey!
--
Best regards,

Roman'''
    eq_(('Hey!', '--\nBest regards,\n\nRoman'),
        bruteforce.extract_signature(msg_body))

    msg_body = '''Hey!
--
--
Regards,
Roman'''
    eq_(('Hey!', '--\n--\nRegards,\nRoman'),
        bruteforce.extract_signature(msg_body))


def test_iphone_signature():
    msg_body = '''Hey!

Sent from my iPhone!'''
    eq_(('Hey!', 'Sent from my iPhone!'),
        bruteforce.extract_signature(msg_body))


def test_mailbox_for_iphone_signature():
    msg_body = """Blah
Sent from Mailbox for iPhone"""
    eq_(("Blah", "Sent from Mailbox for iPhone"),
        bruteforce.extract_signature(msg_body))


def test_line_starts_with_signature_word():
    msg_body = '''Hey man!
Thanks for your attention.
--
Thanks!
Roman'''
    eq_(('Hey man!\nThanks for your attention.', '--\nThanks!\nRoman'),
        bruteforce.extract_signature(msg_body))


def test_line_starts_with_dashes():
    msg_body = '''Hey man!
Look at this:

--> one
--> two
--
Roman'''
    eq_(('Hey man!\nLook at this:\n\n--> one\n--> two', '--\nRoman'),
        bruteforce.extract_signature(msg_body))


def test_blank_lines_inside_signature():
    msg_body = '''Blah.

-Lev.

Sent from my HTC smartphone!'''
    eq_(('Blah.', '-Lev.\n\nSent from my HTC smartphone!'),
        bruteforce.extract_signature(msg_body))

    msg_body = '''Blah
--

John Doe'''
    eq_(('Blah', '--\n\nJohn Doe'), bruteforce.extract_signature(msg_body))


def test_blackberry_signature():
    msg_body = """Heeyyoooo.
Sent wirelessly from my BlackBerry device on the Bell network.
Envoyé sans fil par mon terminal mobile BlackBerry sur le réseau de Bell."""
    eq_(('Heeyyoooo.', msg_body[len('Heeyyoooo.\n'):]),
        bruteforce.extract_signature(msg_body))

    msg_body = u"""Blah
Enviado desde mi oficina mÃ³vil BlackBerryÂ® de Telcel"""

    eq_(('Blah', u'Enviado desde mi oficina mÃ³vil BlackBerryÂ® de Telcel'),
        bruteforce.extract_signature(msg_body))


@patch.object(bruteforce, 'get_delimiter', Mock(side_effect=Exception()))
def test_crash_in_extract_signature():
    msg_body = '''Hey!
-roman'''
    eq_((msg_body, None), bruteforce.extract_signature(msg_body))


def test_signature_cant_start_from_first_line():
    msg_body = """Thanks,

Blah

regards

John Doe"""
    eq_(('Thanks,\n\nBlah', 'regards\n\nJohn Doe'),
        bruteforce.extract_signature(msg_body))


@patch.object(bruteforce, 'SIGNATURE_MAX_LINES', 2)
def test_signature_max_lines_ignores_empty_lines():
    msg_body = """Thanks,
Blah

regards


John Doe"""
    eq_(('Thanks,\nBlah', 'regards\n\n\nJohn Doe'),
        bruteforce.extract_signature(msg_body))


def test_get_signature_candidate():
    # if there aren't at least 2 non-empty lines there should be no signature
    for lines in [], [''], ['', ''], ['abc']:
        eq_([], bruteforce.get_signature_candidate(lines))

    # first line never included
    lines = ['text', 'signature']
    eq_(['signature'], bruteforce.get_signature_candidate(lines))

    # test when message is shorter then SIGNATURE_MAX_LINES
    with patch.object(bruteforce, 'SIGNATURE_MAX_LINES', 3):
        lines = ['text', '', '', 'signature']
        eq_(['signature'], bruteforce.get_signature_candidate(lines))

    # test when message is longer then the SIGNATURE_MAX_LINES
    with patch.object(bruteforce, 'SIGNATURE_MAX_LINES', 2):
        lines = ['text1', 'text2', 'signature1', '', 'signature2']
        eq_(['signature1', '', 'signature2'],
            bruteforce.get_signature_candidate(lines))

    # test long lines not encluded
    with patch.object(bruteforce, 'TOO_LONG_SIGNATURE_LINE', 3):
        lines = ['BR,', 'long', 'Bob']
        eq_(['Bob'], bruteforce.get_signature_candidate(lines))

    # test list (with dashes as bullet points) not included
    lines = ['List:,', '- item 1', '- item 2', '--', 'Bob']
    eq_(['--', 'Bob'], bruteforce.get_signature_candidate(lines))


def test_mark_candidate_indexes():
    with patch.object(bruteforce, 'TOO_LONG_SIGNATURE_LINE', 3):
        # spaces are not considered when checking line length
        eq_('clc',
            bruteforce._mark_candidate_indexes(
                ['BR,  ', 'long', 'Bob'],
                [0, 1, 2]))

        # only candidate lines are marked
        # if line has only dashes it's a candidate line
        eq_('ccdc',
            bruteforce._mark_candidate_indexes(
                ['-', 'long', '-', '- i', 'Bob'],
                [0, 2, 3, 4]))


def test_process_marked_candidate_indexes():
    eq_([2, 13, 15],
        bruteforce._process_marked_candidate_indexes(
            [2, 13, 15], 'dcc'))

    eq_([15],
        bruteforce._process_marked_candidate_indexes(
            [2, 13, 15], 'ddc'))

    eq_([13, 15],
        bruteforce._process_marked_candidate_indexes(
            [13, 15], 'cc'))

    eq_([15],
        bruteforce._process_marked_candidate_indexes(
            [15], 'lc'))

    eq_([15],
        bruteforce._process_marked_candidate_indexes(
            [13, 15], 'ld'))



# -*- coding: utf-8 -*-

"""
The module's functions operate on message bodies trying to extract original
messages (without quoted messages) from html
"""

import regex as re


CHECKPOINT_PREFIX = '#!%!'
CHECKPOINT_SUFFIX = '!%!#'
CHECKPOINT_PATTERN = re.compile(CHECKPOINT_PREFIX + '\d+' + CHECKPOINT_SUFFIX)

# HTML quote indicators (tag ids)
QUOTE_IDS = ['OLK_SRC_BODY_SECTION']


def add_checkpoint(html_note, counter):
    """Recursively adds checkpoints to html tree.
    """
    if html_note.text:
        html_note.text = (html_note.text + CHECKPOINT_PREFIX +
                          str(counter) + CHECKPOINT_SUFFIX)
    else:
        html_note.text = (CHECKPOINT_PREFIX + str(counter) +
                          CHECKPOINT_SUFFIX)
    counter += 1

    for child in html_note.iterchildren():
        counter = add_checkpoint(child, counter)

    if html_note.tail:
        html_note.tail = (html_note.tail + CHECKPOINT_PREFIX +
                          str(counter) + CHECKPOINT_SUFFIX)
    else:
        html_note.tail = (CHECKPOINT_PREFIX + str(counter) +
                          CHECKPOINT_SUFFIX)
    counter += 1

    return counter


def delete_quotation_tags(html_note, counter, quotation_checkpoints):
    """Deletes tags with quotation checkpoints from html tree.
    """
    tag_in_quotation = True

    if quotation_checkpoints[counter]:
        html_note.text = ''
    else:
        tag_in_quotation = False
    counter += 1

    quotation_children = []  # Children tags which are in quotation.
    for child in html_note.iterchildren():
        counter, child_tag_in_quotation = delete_quotation_tags(
            child, counter,
            quotation_checkpoints
        )
        if child_tag_in_quotation:
            quotation_children.append(child)

    if quotation_checkpoints[counter]:
        html_note.tail = ''
    else:
        tag_in_quotation = False
    counter += 1

    if tag_in_quotation:
        return counter, tag_in_quotation
    else:
        # Remove quotation children.
        for child in quotation_children:
            html_note.remove(child)
        return counter, tag_in_quotation


def cut_gmail_quote(html_message):
    ''' Cuts the outermost block element with class gmail_quote. '''
    gmail_quote = html_message.cssselect('.gmail_quote')
    if gmail_quote:
        gmail_quote[0].getparent().remove(gmail_quote[0])
        return True


def cut_microsoft_quote(html_message):
    ''' Cuts splitter block and all following blocks. '''
    splitter = html_message.xpath(
        #outlook 2007, 2010
        "//div[@style='border:none;border-top:solid #B5C4DF 1.0pt;"
        "padding:3.0pt 0cm 0cm 0cm']|"
        #windows mail
        "//div[@style='padding-top: 5px; "
        "border-top-color: rgb(229, 229, 229); "
        "border-top-width: 1px; border-top-style: solid;']"
    )

    if splitter:
        splitter = splitter[0]
        #outlook 2010
        if splitter == splitter.getparent().getchildren()[0]:
            splitter = splitter.getparent()
    else:
        #outlook 2003
        splitter = html_message.xpath(
            "//div"
            "/div[@class='MsoNormal' and @align='center' "
            "and @style='text-align:center']"
            "/font"
            "/span"
            "/hr[@size='3' and @width='100%' and @align='center' "
            "and @tabindex='-1']"
        )
        if len(splitter):
            splitter = splitter[0]
            splitter = splitter.getparent().getparent()
            splitter = splitter.getparent().getparent()

    if len(splitter):
        parent = splitter.getparent()
        after_splitter = splitter.getnext()
        while after_splitter is not None:
            parent.remove(after_splitter)
            after_splitter = splitter.getnext()
        parent.remove(splitter)
        return True

    return False


def cut_by_id(html_message):
    found = False
    for quote_id in QUOTE_IDS:
        quote = html_message.cssselect('#{}'.format(quote_id))
        if quote:
            found = True
            quote[0].getparent().remove(quote[0])
    return found


def cut_blockquote(html_message):
    ''' Cuts blockquote with wrapping elements. '''
    quote = html_message.find('.//blockquote')
    if quote is not None:
        quote.getparent().remove(quote)
        return True


def cut_from_block(html_message):
    """Cuts div tag which wraps block starting with "From:"."""
    # handle the case when From: block is enclosed in some tag
    block = html_message.xpath(
        ("//*[starts-with(mg:text_content(), 'From:')]|"
         "//*[starts-with(mg:text_content(), 'Date:')]"))

    if block:
        block = block[-1]
        while block.getparent() is not None:
            if block.tag == 'div':
                block.getparent().remove(block)
                return True
            else:
                block = block.getparent()
    else:
        # handle the case when From: block goes right after e.g. <hr>
        # and not enclosed in some tag
        block = html_message.xpath(
            ("//*[starts-with(mg:tail(), 'From:')]|"
             "//*[starts-with(mg:tail(), 'Date:')]"))
        if block:
            block = block[0]
            while(block.getnext() is not None):
                block.getparent().remove(block.getnext())
            block.getparent().remove(block)
            return True



import os

from setuptools import setup, find_packages


setup(
    name='claw',
    version='1.0.0',
    description='Library to extract message quotations and signatures.',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author='Adam Renberg',
    author_email='tgwizard@gmail.com',
    url='https://github.com/tgwizard/claw',
    license='APACHE2',
    packages=find_packages(exclude=['tests']),
    zip_safe=True,
    install_requires=[
        'lxml==2.3.3',
        'regex>=2015.07.19',
        'html2text',
        'setuptools>=17.1',
    ],
    extras_require={
        'tests': [
            'pytest==2.7.2',
            'nose==1.3.7',
            'mock==1.3.0',
            'coverage==3.7.1',
            'flanker',
            'tox',
        ],
        'dev': [
            'twine',
        ]
    },
)
